/*
 *                            *
 *                           * *
 *                          *   *
 *                     ***************
 *                      * *       * *
 *                       *  MUMPS  *
 *                      * *       * *
 *                     ***************
 *                          *   *
 *                           * *
 *                            *
 *
 *   ssvn_device.c
 *    ^$DEVICE SSV support
 *
 *  
 *   Author: John P. Willis <jpw@coherent-logic.com>
 *    Copyright (C) 1998 MUG Deutschland
 *    Copyright (C) 2020 Coherent Logic Development LLC
 *
 **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "mref.h"
#include "mpsdef.h"

void ssvn_device(short action, char *key, char *data)
{
    int channel;
    int fd;
    int i;
    FILE *fp;

    freem_ref_t *r = (freem_ref_t *) malloc (sizeof (freem_ref_t));
    char *dbuf = (char *) malloc (100 * sizeof (char));

    mref_init (r, MREF_RT_SSV, "^$DEVICE");
    internal_to_mref (r, key);

    if (r->subscript_count < 2) {
        ierr = INVREF;
        goto done;
    }

    if (!isdigit (r->subscripts[0][0])) {
        ierr = NODEVICE;
        goto done;
    }

    channel = atoi (r->subscripts[0]);

    if (channel < 0 || channel >= MAXDEV) {
        ierr = NODEVICE;
        goto done;
    }

    fp = opnfile[channel];

    if (channel != 0 && (fp == NULL || (fd = fileno (fp)) == -1) && channel < FIRSTSCK) {
        ierr = NOPEN;
        goto done;
    }

    for (i = 0; i < strlen (r->subscripts[1]); i++) r->subscripts[1][i] = toupper (r->subscripts[1][i]);

    key = mref_to_internal (r);

    switch (action) {

        case get_sym:

            if (strcmp (r->subscripts[1], "EOF") == 0 && channel != 0) {

                if (feof (fp)) {
                    sprintf (data, "1\201");
                }
                else {
                    sprintf (data, "0\201");
                }

		break;
		
            }
            else if (strcmp (r->subscripts[1], "LENGTH") == 0 && channel != 0) {

                struct stat s;
                off_t siz;

                fstat (fd, &s);
                siz = s.st_size;

                sprintf (data, "%d\201", siz);  

                break;  

            }    
            else if (strcmp (r->subscripts[1], "MNEMONICSPACE") == 0) {
                
                if (r->subscript_count == 2) {

                    switch (channel) {
                        
                        case 0:
                            sprintf (data, "X364\201");
                            break;
                    
                        default:
                            sprintf (data, "\201");
                            break;
                    }

                }
                else {
                    ierr = INVREF;
                    goto done;
                }

                break;
            }    
            else if (strcmp (r->subscripts[1], "DSW") == 0 && channel == 0) {

                sprintf (data, "%d\201", DSW);

                ierr = OK;
                goto done;

            }     
            else if (strcmp (r->subscripts[1], "TERMINATOR") == 0) {
                symtab (get_sym, key, data);
            }
            else {
                ierr = SBSCR;
                goto done;
            }

            break;

        case set_sym:

            if (strcmp (r->subscripts[1], "DSW") == 0 && channel == 0) {

                stcpy (dbuf, data);
                stcnv_m2c (dbuf);

                DSW = atol (dbuf);

                ierr = OK;
                goto done;

            }
            else if (strcmp (r->subscripts[1], "TERMINATOR") == 0) {
                symtab (set_sym, key, data);
            }
            else {
                ierr = M29;
                goto done;
            }

            break;


        default:
            ierr = INVREF;
            break;
    }

done:

    free (r);
    free (dbuf);

    return;
}
